//---------------------------------------------------------------------------
#ifndef kviewH
#define kviewH
//---------------------------------------------------------------------------
#include <vcl\Classes.hpp>
#include <vcl\Controls.hpp>
#include <vcl\StdCtrls.hpp>
#include <vcl\Forms.hpp>
#include <vcl\ComCtrls.hpp>
#include "Grids.hpp"
//---------------------------------------------------------------------------

#include <mk4.h>
#include <mk4io.h>
#include <mk4str.h>
#include <vcl\Dialogs.hpp>
#include <vcl\ExtCtrls.hpp>

class ViewerStrategy : public c4_FileStrategy
{
    int _offset; // starting point of datafile
public:
    c4_String _fileName;

public:
    ViewerStrategy (FILE* file_ =0)
        : c4_FileStrategy (file_), _offset (0)
    {
    }

    virtual ~ViewerStrategy ()
    {
        if (_mapStart != 0)
            _mapStart -= _offset;
    }

        // the other members have overrides to allow datafiles at >0 offsets

    virtual bool DataOpen(const char* fileName_, bool allowWriting_)
    {
        _fileName = fileName_;

        bool exists = c4_FileStrategy::DataOpen(fileName_, allowWriting_);

        if (exists && IsValid()) {
            ResetFileMapping();

                // look for the magic header bytes in the first 4 Kb
                // of the file, but only check on 16-byte boundaries
            
                // seek to start, since ResetFileMapping may have moved it!
            DataSeek(0);
            
            char buffer [4096];
            const int n = DataRead(buffer, sizeof buffer);

            for (const char* p = buffer; p < buffer + n; p += 16)
            {
                if (p[2] == '\x1A' && p[3] == '\x80' &&
                        (p[0] == '\x4A' && p[1] == '\x4C' ||
                         p[0] == '\x4C' && p[1] == '\x4A'))
                {
                    _offset = (int) (p - buffer);

                        // adjust file mapping if it is on
                    if (_mapStart != 0)
                        _mapStart += _offset;

                    break;
                }
            }
        }
        
        _cleanup = _file;

        return exists;
    }

    virtual void DataSeek(t4_i32 position_)
    {
        if (IsValid())
            c4_FileStrategy::DataSeek(position_ + _offset);
    }

    virtual int DataRead(void* buffer_, int length_)
    {
            // accept invalid file handle for in-memory use
        return IsValid() ? c4_FileStrategy::DataRead(buffer_, length_) : 0;
    }

    virtual void DataCommit(t4_i32 newSize_)
    {
        c4_FileStrategy::DataCommit(newSize_ > 0 ? newSize_ + _offset : 0);
    }

    virtual void ResetFileMapping()
    {
        if (_mapStart != 0)
            _mapStart -= _offset;

        c4_FileStrategy::ResetFileMapping();

        if (_mapStart != 0)
            _mapStart += _offset;
    }
};

class TMainForm : public TForm
{
__published:	// IDE-managed Components
    TOpenDialog *OpenDialog;
    TTreeView *StructTree;
    TSplitter *Splitter;
    TDrawGrid *DataGrid;

	void __fastcall StructTreeChange(TObject *Sender, TTreeNode *Node);
	
	
	
	void __fastcall FormClose(TObject *Sender, TCloseAction &Action);
    void __fastcall FormActivate(TObject *Sender);
    void __fastcall DataGridDrawCell(TObject *Sender, int ACol, int ARow,
          TRect &Rect, TGridDrawState State);
	
	
private:	// User declarations

    ViewerStrategy _strategy;
	c4_Storage* _storage;

    	// defines the current data view
    c4_View _path;		// used as a stack
    c4_View _data;		// the top view, currently visible
    int _colNum;		// >= 0 to show a single column, else -1

	void __fastcall SetupTree(TTreeNode* node_, c4_View& view_, int index_ =0);
	void __fastcall SetupData();

public:		// User declarations
	__fastcall TMainForm(TComponent* Owner);
};
//---------------------------------------------------------------------------
extern TMainForm *MainForm;
//---------------------------------------------------------------------------
#endif
